package com.clx.performance.service.impl;

import com.clx.performance.enums.OrderChildEnum;
import com.clx.performance.app.SaveOrderChildParam;
import com.clx.performance.app.SaveOrderChildVo;
import com.clx.performance.dao.OrderChildDao;
import com.clx.performance.dao.OrderGoodsDao;
import com.clx.performance.model.OrderChild;
import com.clx.performance.model.OrderGoods;
import com.clx.performance.service.OrderChildLogService;
import com.clx.performance.service.OrderChildService;
import com.msl.common.enums.ResultCodeEnum;
import com.msl.common.exception.ServiceSystemException;
import com.msl.user.data.UserSessionData;
import com.msl.user.utils.TokenUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;

/**
 * @Author: aiqinguo
 * @Description: 运单表
 * @Date: 2023/09/18 11:34:50
 * @Version: 1.0
 */

@Slf4j
@Service
@AllArgsConstructor
public class OrderChildServiceImpl implements OrderChildService {

    private final OrderChildDao orderChildDao;

    private final OrderGoodsDao orderGoodsDao;

    private final OrderChildLogService orderChildLogService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public SaveOrderChildVo saveOrderChild(SaveOrderChildParam param) {
        UserSessionData loginUserInfo = TokenUtil.getLoginUserInfo();

        Long userNo = loginUserInfo.getUserNo();

        String childNo = "1";

        BigDecimal truckLoad = BigDecimal.valueOf(1L);

        LocalDateTime now = LocalDateTime.now();
        OrderGoods orderGoods = orderGoodsDao.getByOrderGoodsNo(param.getOrderGoodsNo()).orElseThrow(ResultCodeEnum.FAIL);
        // 库存验证
        if (orderGoods.getResidueTransportWeight().compareTo(BigDecimal.ZERO) <= 0) {throw new ServiceSystemException(ResultCodeEnum.FAIL);}
        if (orderGoods.getResidueTransportWeight().compareTo(truckLoad) < 0) {throw new ServiceSystemException(ResultCodeEnum.FAIL);}

        OrderChild orderChild = new OrderChild();
        orderChild.setChildNo(childNo);
        orderChild.setUserNo(userNo);
        orderChild.setOwnerUserNo(orderGoods.getUserNo());
        orderChild.setOrderNo(orderGoods.getOrderNo());
        orderChild.setOrderGoodsNo(orderGoods.getOrderGoodsNo());
        orderChild.setFreightPrice(orderGoods.getPendingOrderFreight());

        orderChild.setGoodsId(0);
        orderChild.setGoodsName(orderGoods.getGoodsName());

        orderChild.setSendAddressId(orderGoods.getSendAddressId());
        orderChild.setSendAddress(orderGoods.getSendAddressShorter());
        orderChild.setReceiveAddressId(orderGoods.getReceiveAddressId());
        orderChild.setReceiveAddress(orderGoods.getReceiveAddressShorter());

        orderChild.setDriverUserNo(param.getDriverUserNo());
        orderChild.setDriverName("");
        orderChild.setDriverMobile("");

        orderChild.setTruckId(param.getTruckId());
        orderChild.setTruckNo("");
        orderChild.setTruckLoad(truckLoad);

        orderChild.setWeight(orderChild.getTruckLoad());
        orderChild.setPayTime(now);
        orderChild.setStatus(OrderChildEnum.Status.CREATED.getCode());
        orderChild.setCreateTime(now);

        // 更新货单数据
        updateOrderGoodsAmount(orderGoods, truckLoad);

        // 新增运单
        orderChildDao.saveEntity(orderChild);

        // 更新司机状态

        // 更新车辆状态

        // 新增日志
        orderChildLogService.saveOrderChildLog(childNo, 1,"创建运单",
                1, loginUserInfo.getUserNo(), loginUserInfo.getUserName());

        SaveOrderChildVo result = new SaveOrderChildVo();
        result.setChildNo(childNo);
        return result;
    }

    /**
     * 更新货单数据
     */
    private void updateOrderGoodsAmount(OrderGoods orderGoods, BigDecimal weight){
        if (orderGoods.getResidueTransportWeight().compareTo(weight) <= 0) {orderGoods.setOrderGoodsStatus(0);}
//        orderGoodsDao.updateEntityByKey(orderGoods, weight);
    }

}
